@@ -12,11 +12,13 @@ use thiserror::Error;
 use url::Url;
 
 pub use numeric::*;
+pub use page::*;
 pub use persons::*;
 pub use strings::*;
 pub use time::*;
 
 mod numeric;
+mod page;
 mod persons;
 mod strings;
 mod time;
@@ -302,6 +304,16 @@ pub enum MaybeTyped<T> {
     String(String),
 }
 
+impl<T> MaybeTyped<T> {
+    /// Get the typed value, if it is present.
+    pub fn as_typed(&self) -> Option<&T> {
+        match self {
+            MaybeTyped::Typed(t) => Some(t),
+            MaybeTyped::String(_) => None,
+        }
+    }
+}
+
 impl<T: ToOwned> MaybeTyped<T> {
     /// Wrap the typed value in a [`Cow`]'s borrowed variant.
     pub fn to_cow(&self) -> MaybeTyped<Cow<T>> {
@@ -409,6 +421,59 @@ impl Display for QualifiedUrl {
     }
 }
 
+derive_or_from_str! {
+    /// A publisher, possibly with a location.
+    #[derive(Clone, Debug, PartialEq, Eq, Hash)]
+    pub struct Publisher where "FormatString string or dictionary with \"name\" and \"location\"" {
+        /// Publisher of the item.
+        name: Option<FormatString>,
+        /// Physical location at which the item was published or created.
+        location: Option<FormatString>,
+    }
+}
+
+impl Serialize for Publisher {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        if let Some(location) = &self.location {
+            let mut map = serializer.serialize_map(Some(2))?;
+            map.serialize_entry("name", &self.name)?;
+            map.serialize_entry("location", location)?;
+            map.end()
+        } else {
+            self.name.serialize(serializer)
+        }
+    }
+}
+
+impl Publisher {
+    /// Create a new publisher.
+    pub fn new(name: Option<FormatString>, location: Option<FormatString>) -> Self {
+        Self { name, location }
+    }
+
+    /// Publisher of the item.
+    pub fn name(&self) -> Option<&FormatString> {
+        self.name.as_ref()
+    }
+
+    /// Physical location at which the item was published or created.
+    pub fn location(&self) -> Option<&FormatString> {
+        self.location.as_ref()
+    }
+}
+
+impl FromStr for Publisher {
+    type Err = ChunkedStrParseError;
+
+    /// Creates a new publisher with `s` as its name and no location.
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        Ok(Publisher::new(Some(FormatString::from_str(s)?), None))
+    }
+}
+
 /// A set of serial numbers like DOIs, ISBNs, or ISSNs.
 /// Keys should be lowercase.
 #[derive(Clone, Debug, PartialEq, Eq, Serialize, Hash)]
@@ -436,13 +501,13 @@ impl<'de> Deserialize<'de> for SerialNumber {
             Float(f64),
         }
 
-        impl ToString for StringOrNumber {
-            fn to_string(&self) -> String {
+        impl Display for StringOrNumber {
+            fn fmt(&self, formatter: &mut fmt::Formatter<'_>) -> fmt::Result {
                 match self {
-                    Self::String(s) => s.clone(),
-                    Self::Number(n) => n.to_string(),
-                    Self::UnsignedNumber(n) => n.to_string(),
-                    Self::Float(f) => f.to_string(),
+                    Self::String(s) => s.fmt(formatter),
+                    Self::Number(n) => n.fmt(formatter),
+                    Self::UnsignedNumber(n) => n.fmt(formatter),
+                    Self::Float(f) => f.fmt(formatter),
                 }
             }
         }
